/*
 * Support for Bounds checking nbq functions
 */

#ifdef NETBUF_PROXY

#include <stddef.h>
#include <sys/types.h>
#if defined(m68k)
#import "spl.h"
#else
#import <kernserv/machine/spl.h>
#endif
#include <net/ppp_defs.h>
#include "nbq.h"

/*
 * There is no driver kit for the Moto release.
 */
#ifndef IOLog
#define IOLog printf
#define	IOLogDbg		if (sc->sc_flags & SC_DEBUG) printf
#else
#define	IOLogDbg		if (sc->sc_flags & SC_DEBUG) IOLog
#endif


#define Z_ALIGN 3	/* 8 byte boudary */
#define Z_EXTRA ((unsigned long)((1<<Z_ALIGN)-1))
#define ALIGN(x) ((unsigned long)(x+Z_EXTRA) & (unsigned long)(~Z_EXTRA))

#define BOUNDS_SIZE 4
#define BOUNDS_BEGIN 0xfeedface
#define BOUNDS_END   0xcafebabe

/*
 * kalloc returns unaligned memory.  Make sure we
 * align it here.  We have to store orignal address
 * and size for freeing.
 */
ppp_netbuf_t cb_nb_alloc(unsigned size)
{

   char *u_buf;
   ppp_netbuf_t buf;

   u_buf = kalloc(sizeof(struct _ppp_netbuf_t) + Z_EXTRA);
   buf = (ppp_netbuf_t) ALIGN(u_buf);

   buf->wrapper = u_buf;
   buf->next=NULL;

#ifdef CHECK_BOUNDS
   size += (2 * BOUNDS_SIZE);
#endif

   buf->buffer = nb_alloc(size);
   buf->orig_buffer = NULL;
   buf->init_offset = nb_map(buf->buffer);
   buf->size = size;
   bzero(&buf->pktinfo, sizeof(bpf_encapsulater));

#ifdef CHECK_BOUNDS
#warning CHECK_BOUNDS enabled

   /*
    * Write bounds checking info
    */
   {
     unsigned int bound;

     bound = BOUNDS_BEGIN;
     bcopy(&bound, buf->init_offset, BOUNDS_SIZE);
     bound = BOUNDS_END;
     bcopy(&bound, (buf->init_offset + size) - BOUNDS_SIZE, BOUNDS_SIZE);
     nb_shrink_top(buf->buffer, BOUNDS_SIZE);
     nb_shrink_bot(buf->buffer, BOUNDS_SIZE);
   }
#endif  /* CHECK_BOUNDS */

   return buf;
}
     
void cb_nb_free(ppp_netbuf_t nb)
{
  if (nb)
    {
      if (nb->buffer)
	nb_free(nb->buffer);
      if (nb->orig_buffer)
	nb_free(nb->orig_buffer);
      kfree(nb->wrapper, sizeof(struct _ppp_netbuf_t) + Z_EXTRA);
    }
  else
    IOLog("Received empty ppp_netbuf_t at %s %d\n", __FILE__, __LINE__);
  return;
}

void cb_nb_duplicate(ppp_netbuf_t from, ppp_netbuf_t to)
{
  /*
   * Copy all the interesting info from the 'from' netbuf
   * to the 'to' netbuf.  Usually called after BSD compress
   * returns new "compressed" netbuf.
   */

  to->orig_buffer = from->orig_buffer;
  from->orig_buffer = NULL;

  bcopy(&(from->pktinfo), &(to->pktinfo), sizeof (bpf_encapsulater));
  
}

char * cb_nb_map(ppp_netbuf_t nb)
{
  return nb_map(nb->buffer);
}

char * cb_nb_map_orig(ppp_netbuf_t nb)
{
  if (nb->orig_buffer)
    return nb_map(nb->orig_buffer);
  else
    {
      IOLog ("Warning: returning bad orig at %s %d\n", __FILE__, __LINE__);
      return nb_map(nb->buffer);
    }
}

unsigned cb_nb_size(ppp_netbuf_t nb)
{
  return nb_size(nb->buffer);
}

unsigned cb_nb_size_orig(ppp_netbuf_t nb)
{
  if(nb->orig_buffer)
    return nb_size(nb->orig_buffer);
  else
    {
      IOLog ("Warning: returning bad orig size at %s %d\n", __FILE__, __LINE__);
      return nb_size(nb->buffer);
    }
}

int cb_nb_shrink_top(ppp_netbuf_t nb, unsigned size)
{
  return nb_shrink_top(nb->buffer, size);
}

int cb_nb_grow_top(ppp_netbuf_t nb, unsigned size)
{
  return nb_grow_top(nb->buffer, size);
}

int cb_nb_shrink_bot(ppp_netbuf_t nb, unsigned size)
{
  return nb_shrink_bot(nb->buffer, size);
}

int cb_nb_grow_bot(ppp_netbuf_t nb, unsigned size)
{
  return nb_grow_bot(nb->buffer, size);
}

int cb_nb_read(ppp_netbuf_t nb, unsigned offset, unsigned size, void *target)
{
  return nb_read(nb->buffer, offset, size, target);
}

int cb_nb_write(ppp_netbuf_t nb, unsigned offset, unsigned size, void *source)
{
  return nb_write(nb->buffer, offset, size, source);
}

void cb_nb_get_mark(ppp_netbuf_t nb, mark_t *ptr)
{
  bcopy((char *) &nb->mark, (char *) ptr, sizeof(mark_t));
}

void cb_nb_set_mark(ppp_netbuf_t nb, mark_t ptr)
{
  bcopy((char *) &ptr, &nb->mark, sizeof(mark_t));
}

void cb_nb_get_next(ppp_netbuf_t nb, ppp_netbuf_t *ptr)
{
  bcopy(&nb->next, ptr, sizeof(ppp_netbuf_t));
}

void cb_nb_set_next(ppp_netbuf_t nb, ppp_netbuf_t ptr)
{
  bcopy(&ptr, &nb->next, sizeof(ppp_netbuf_t));
}

/*
 * Careful about writing bounds info here.  We lose info
 * on a conversion of NB to nb so this reverse conversion
 * can cause problems.  However, at the time of writing, there
 * are only 2 calls to this funcation and both operate directly
 * on netbufs that were allocated by pppgetbuf().  So, we make the
 * assumption that the top and bottom have been shrunk as was
 * done in pppgetbuf(). So, if we unshrink the top and bottom
 * then we have a netbuf_t that was returned by NB_ALLOC.
 *
 * Since both of these calls to this function are used in
 * outgoing procedures (packets either received from the user
 * level pppd or from the TCP/IP stack and destined for the
 * peer) we will stash a copy of the original datagram here
 * so that we can forward it to BPF if requested.
 *
 */

ppp_netbuf_t cb_nb_to_NB(netbuf_t nb)
{
   char *u_buf;
   ppp_netbuf_t buf;

   u_buf = kalloc(sizeof(struct _ppp_netbuf_t) + Z_EXTRA);
   buf = (ppp_netbuf_t) ALIGN(u_buf);

   buf->wrapper = u_buf;
   buf->buffer = nb;   /* previously correctly allocated */
   buf->next = NULL;
   buf->init_offset = nb_map(buf->buffer) - PPP_HDRLEN;

   /*
    * Must add back in the space that pppgetbuf()
    * adds.
    */
   buf->size = nb_size(nb) +  PPP_HDRLEN + PPP_FCSLEN;

   /*
    * Stash a copy of the original netbuf so that BPF
    * can handle outgoing buffers.  We shouldn't care
    * About CHECK_BOUNDS info or FCSLEN in this packet
    */

   buf->orig_buffer = nb_alloc(nb_size(nb) + PPP_HDRLEN);
   bcopy(nb_map(nb), nb_map(buf->orig_buffer) + PPP_HDRLEN, nb_size(nb));
   

#ifdef CHECK_BOUNDS

   buf->init_offset -= BOUNDS_SIZE;
   buf->size += (2 * BOUNDS_SIZE);

#endif


   return buf;
}

/*
 * Here we just toss the container and return the
 * actual netbuf.  We lose information in this operation.
 */
netbuf_t cb_NB_to_nb(ppp_netbuf_t nb)
{
  netbuf_t tmp;

#ifdef CHECK_BOUNDS
  {
    unsigned int bound;

    bound = BOUNDS_BEGIN;
    if (bcmp(nb->init_offset, &bound, BOUNDS_SIZE) != 0)
      IOLog ("Overwrote beginning of netbuf\n");
    
    bound = BOUNDS_END;
    if (bcmp((nb->init_offset + nb->size) - BOUNDS_SIZE, &bound, BOUNDS_SIZE) != 0)
      IOLog ("Overwrote end of netbuf\n");    
  }
#endif  

  tmp = nb->buffer;
  if (nb->orig_buffer)
    nb_free(nb->orig_buffer);

  kfree(nb->wrapper, sizeof(struct _ppp_netbuf_t) + Z_EXTRA);
  return tmp;
}



#endif /* NETBUF_PROXY */
